VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Escer"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit
'=====
'Escer
'=====
'
'A VB6 class providing two methods intended to help make entry
'and display of String values containing control characters or
'Unicode easier.
'
'A simple C-like convention is used although as written the
'escaping isn't exactly the same as in C, but tailored to fit VB.
'
'This is a "PredeclaredId" class.  There is a global instance
'of this class named "Escer" provided automatically.
'

Private Enum UnEscStates
    uesNotInEscapeSeq = 0
    uesProcessingEscSeq = 1
    uesProcessingLongEscSeq = 2
    uesProcessingLongEscSeqXEnd = 3
    uesProcessingLongEscSeqUEnd = 5
End Enum

Public Function Escape(ByVal PlainText As String) As String
Attribute Escape.VB_Description = "Format plain text as escaped text"
    'Escape a string value.  Escape sequences generated:
    '
    '   \xhh   7 bit hex
    '   \uhhhh 16 bit hex ("Unicode")
    '   \b     vbBack
    '   \t     vbTab
    '   \v     vbVerticalTab
    '   \f     vbFormFeed
    '   \r     vbCr
    '   \l     vbLf
    '   \n     vbNewLine/vbCrLf
    '   \q     quote (")
    '   \0     vbNullChar
    '   \\     \
    
    Dim Pos As Long
    Dim ChS As String
    Dim ChW As Integer
    Dim EscCh As String
    Dim EscChLen As Long
    Dim EscPos As Long
    
    'Estimate size required as 150% of input length.  Adjust as required.
    Escape = Space$(Len(PlainText) * 15& \ 10&)
    For Pos = 1& To Len(PlainText)
        ChS = Mid$(PlainText, Pos, 1&)
        ChW = AscW(ChS)
        Select Case ChW
            Case 0#
                EscCh = "\0"
            Case 1# To 7#, 14# To 31#, 127#
                EscCh = "\x" & Right$("0" & Hex$(ChW), 2&)
            Case 8#
                EscCh = "\b"
            Case 9#
                EscCh = "\t"
            Case 10#
                EscCh = "\l"
            Case 11#
                EscCh = "\v"
            Case 12#
                EscCh = "\f"
            Case 13#
                If Pos < Len(PlainText) Then
                    If AscW(Mid$(PlainText, Pos + 1, 1&)) = 10# Then
                        EscCh = "\n"
                        Pos = Pos + 1
                    Else
                        EscCh = "\r"
                    End If
                Else
                    EscCh = "\r"
                End If
            Case 32#, 33#, 35# To 91#, 93# To 126#
                EscCh = ChS
            Case 34#
                EscCh = "\q"
            Case 92#
                EscCh = "\\"
            Case Else
                EscCh = "\u" & Right$("000" & Hex$(ChW), 4&)
        End Select
        EscChLen = Len(EscCh)
        'Add room if needed, do it in chunks.
        If Len(Escape) < EscPos + EscChLen Then Escape = Escape & Space$(30&)
        Mid$(Escape, EscPos + 1, EscChLen) = EscCh
        EscPos = EscPos + EscChLen
    Next
    Escape = Left$(Escape, EscPos)
End Function

Public Function UnEscape(ByVal EscapedText As String) As String
Attribute UnEscape.VB_Description = "Format escaped text as plain text"
    'Un-escape a string value.  Escape sequences supported:
    '
    '   \xhh   7 bit hex
    '   \uhhhh 16 bit hex ("Unicode")
    '   \b     vbBack
    '   \t     vbTab
    '   \v     vbVerticalTab
    '   \f     vbFormFeed
    '   \r     vbCr
    '   \l     vbLf
    '   \n     vbNewLine/vbCrLf
    '   \q     quote (")
    '   \0     vbNullChar
    '   \\     \
    '
    'Letters indicating escape sequences may be upper or lowercase.
    '
    'If an invalid escape sequence is found we raise error 5.
    
    Dim Pos As Long
    Dim ChS As String
    Dim ChW As Integer
    Dim UpChW As Integer
    Dim UnEscState As UnEscStates
    Dim UnPos As Long
    Dim Unicode As Boolean
    Dim Accum As Long
    
    'Estimate output length = input length. Trim back before exit.
    UnEscape = Space$(Len(EscapedText))
    UnPos = 1&
    For Pos = 1& To Len(EscapedText)
        ChS = Mid$(EscapedText, Pos, 1&)
        ChW = AscW(ChS)
        UpChW = AscW(UCase$(ChS))
        Select Case UnEscState
            Case uesNotInEscapeSeq 'Not processing an escape sequence, check for new one.
                If ChW = 92# Then ' "\"
                    UnEscState = uesProcessingEscSeq
                End If
            Case uesProcessingEscSeq 'We're in an escape sequence, check which one.
                'Assume this finishes the sequence until we check further.
                UnEscState = uesNotInEscapeSeq
                Select Case UpChW
                    Case 88# ' "X"
                        UnEscState = uesProcessingLongEscSeq
                    Case 85# ' "U"
                        Unicode = True
                        UnEscState = uesProcessingLongEscSeq
                    Case 66# ' "B"
                        ChS = vbBack
                    Case 84# ' "T"
                        ChS = vbTab
                    Case 86# ' "V"
                        ChS = vbVerticalTab
                    Case 70# ' "F"
                        ChS = vbFormFeed
                    Case 82# ' "R"
                        ChS = vbCr
                    Case 76# ' "L"
                        ChS = vbLf
                    Case 78# ' "N"
                        Mid$(UnEscape, UnPos, 1&) = vbCr
                        UnPos = UnPos + 1 'Add extra here, 2 chars inserted.
                        ChS = vbLf
                    Case 81# ' "Q"
                        ChS = """"
                    Case 48# ' "0"
                        ChS = vbNullChar
                    Case 92# ' "\", i.e. \\
                        'ChS = ChS
                    Case Else
                        'Undefined escape sequence type char.
                        Err.Raise 5 'Invalid procedure call or argument.
                End Select
            Case Else
                'Working on a \x or \u sequence.  UpChW is a hex digit.
                'If \x we require 2 hex digits, if \u we require 4.
                
                If (48# <= UpChW And UpChW <= 57#) Then
                    '0 through 9.
                    Accum = 16& * Accum + (CLng(UpChW) And &HF&)
                ElseIf (65# <= UpChW And UpChW <= 70#) Then
                    'A through F.
                    Accum = 16& * Accum + (CLng(UpChW) - 55&) 'Subtracting Asc("A") + 10 here.
                Else
                    'Invalid hex digit.
                    Err.Raise 5
                End If
                'Advance the UnEscState state for this hex digit.
                UnEscState = UnEscState + 1&
                If Unicode Then
                    If UnEscState > uesProcessingLongEscSeqUEnd Then
                        ChS = ChrW$(Accum)
                        Accum = 0&
                        UnEscState = uesNotInEscapeSeq
                        Unicode = False
                    End If
                Else
                    '\x## only valid through \x7F to avoid ANSI codepage woes.
                    If Accum > 127 Then Err.Raise 5
                    
                    If UnEscState > uesProcessingLongEscSeqXEnd Then
                        ChS = ChrW$(Accum)
                        Accum = 0&
                        UnEscState = uesNotInEscapeSeq
                    End If
                End If
        End Select
        If UnEscState = uesNotInEscapeSeq Then
            Mid$(UnEscape, UnPos, 1&) = ChS
            UnPos = UnPos + 1&
        End If
    Next
    If UnEscState <> uesNotInEscapeSeq Then Err.Raise 5 'Unfinished escape sequence at end.
    UnEscape = Left$(UnEscape, UnPos - 1&)
End Function


